@@ -15,6 +15,8 @@ from .inventoryrestclient import InventoryRESTClient
 from .labelprinterapiclient import get_labelprinter
 
 
+S_UNSET = '(unset)'
+
 try:
     raw_input  # py2
 except NameError:
@@ -24,6 +26,36 @@ except NameError:
         return input()
 
 
+class Inventory:
+    def __init__(self, inventory_rest_client, hdd_id):
+        self._client = inventory_rest_client
+        self._hdd_id = hdd_id
+        self.flush()
+
+    def flush(self):
+        self._data = self._client.get_hdd(self._hdd_id)[0]
+
+    @property
+    def id(self):
+        return self._data['id']
+
+    @property
+    def tag_uid(self):
+        return self._data['tag_uid']
+
+    @property
+    def bay(self):
+        return self._data['bay']
+
+    @property
+    def current_owner(self):
+        return (self._data.get('current_owner') or {}).get('name')
+
+    @property
+    def current_health_status(self):
+        return (self._data.get('current_health') or {}).get('status')
+
+
 class SymlinkNotFound(Exception):
     pass
 
@@ -71,6 +103,27 @@ class DeviceManager(object):
         self._inventory_rest_client.add_status(
             self._hdd_id, status=status, extra_info=extra_info)
 
+    def _set_health(self, title):
+        # title = 'SECURE ERASED' or ...
+        health = (
+            'OK' if self._hdd_dock.hwdata.wear_health_percent >= 100
+            else '{}%'.format(self._hdd_dock.hwdata.wear_health_percent))
+        health_status = f'{health} - {title}'
+        self._inventory_rest_client.add_health_status(
+            self._hdd_id, status=health_status,
+            extra_info='--{}'.format(self._author))
+
+    def quick_erase(self):
+        print('Start quick wiping')
+        self._hdd_dock.quick_erase()
+        self._set_status('QUICK_WIPED', 'Quick wiped at {} --{}'.format(
+            self._location, self._author))
+        try:
+            self._hdd_dock.flush()
+        finally:
+            self._set_health('QUICK WIPED')
+        print('Disk/ssd quick wiped')
+
     def best_erase(self):
         erase_method = self._hdd_dock.get_erase_methods()[0]
         self._erase(erase_method)
@@ -116,6 +169,7 @@ class DeviceManager(object):
                 'BLKDEV_SECURE_WIPED_ERROR', (
                     'Block device secure wipe error at {}: {} --{}'
                     .format(self._location, error, self._author)))
+            self._set_health('ERASE ERROR')
             return
 
         print('Block device securely wiped')
@@ -129,6 +183,7 @@ class DeviceManager(object):
                     'Block device secure wipe {!r} error at {}, non-empty '
                     'samples found --{}'
                     .format(erase_method.name, self._location, self._author)))
+            self._set_health('ERASE ERROR')
             return
 
         print('OK: All samples are empty')
@@ -139,6 +194,10 @@ class DeviceManager(object):
                 .format(
                     erase_method.name, self._location, sample_count,
                     sample_size, self._author)))
+        try:
+            self._hdd_dock.flush()
+        finally:
+            self._set_health('SECURE ERASED')
 
     def _find_non_zero_blocks(self, sample_count, sample_size):
         print(
@@ -195,12 +254,9 @@ def register_disk(**kwargs):
         hdd_id, location=location)
 
     if erase == 'y' or erase == 'Y':
-        print('Start wiping')
-        hdd_dock.quick_erase()
-        inventory_rest_client.add_status(
-            hdd_id, status='QUICK_WIPED',
-            extra_info='Quick wiped at {} --{}'.format(location, author))
-        print('Disk wiped')
+        manager = DeviceManager(
+            hdd_dock, hdd_id, inventory_rest_client, author, location)
+        manager.quick_erase()
 
     # Print label, unless the disk is in a remote machine
     if location != 'remote':
@@ -218,6 +274,7 @@ def registered_disk_actions(**kwargs):
     hdd_id = kwargs.pop('hdd_id')
     inventory_rest_client = kwargs.pop('inventory_rest_client')
     static_data = kwargs.pop('static_data')
+    del static_data
     dynamic_data = kwargs.pop('dynamic_data')
     author = kwargs.pop('author')
     location = kwargs.pop('location')
@@ -236,43 +293,24 @@ def registered_disk_actions(**kwargs):
             inventory_rest_client.add_location(
                 hdd_id, location=location)
 
-    hdd = inventory_rest_client.get_hdd(hdd_id)[0]
-    current_owner = hdd.get('current_owner', None)
+    inventory = Inventory(inventory_rest_client, hdd_id)
 
-    if current_owner:
-        current_owner = current_owner.get('name', None)
-    if not current_owner:
-        current_owner = 'Notset'
-
-    # current_status = hdd.get('status', None)
-
-    # if current_status:
-    #     current_status = current_status.get('name', None)
-    # if not current_status:
-    #     current_status = 'Notset'
-
-    current_health_status = hdd.get('current_health', None)
-
-    if current_health_status:
-        current_health_status = current_health_status.get('status', None)
-
-    if not current_health_status:
-        current_health_status = 'Notset'
-
-    print_data = static_data + dynamic_data
-
-    print_data.append(['', '-'])
-    print_data.append(['REGISTRATION INFORMATION', '-'])
-    print_data.append(['=', '-'])
-    print_data.append(['Disk is already registered as', hdd['id']])
-    print_data.append(['Last owner', current_owner])
-    # print_data.append(['Last status', current_status])
-    print_data.append(['Last health status', current_health_status])
-    print_data.append(['Server bay', hdd['bay'] or '-'])
+    def _build_print_data():
+        static_data, dynamic_data = build_hdd_info(hdd_dock)
+        print_data = static_data + dynamic_data
+        print_data.append(['', None])
+        print_data.append(['REGISTRATION INFORMATION', None])
+        print_data.append(['=', None])
+        print_data.append(['Disk is already registered as', inventory.id])
+        print_data.append(['Last owner', inventory.current_owner or S_UNSET])
+        print_data.append([
+            'Last health status', inventory.current_health_status or S_UNSET])
+        print_data.append(['Server bay', inventory.bay or S_UNSET])
+        return print_data
 
     manager = DeviceManager(
         hdd_dock, hdd_id, inventory_rest_client, author, location)
-    manager.show_summary(print_data)  # XXX: should not pass print_data here
+    manager.show_summary(_build_print_data())
     manager.show_commands()
 
     while True:
@@ -284,54 +322,43 @@ def registered_disk_actions(**kwargs):
                 owner = raw_input('New owner: ')
             inventory_rest_client.add_owner(  # XXX: not email-of-owner
                 hdd_id, name=owner, email=author)
-            current_owner = owner
+            inventory.flush()
 
         elif action == '2':
             # Print label
             labelprinter_rest_client = get_labelprinter()
             labelprinter_rest_client.print_hdd_label(
-                hdd['tag_uid'],
-                hdd_dock.get_serial_number(),
-                current_owner)
+                inventory.tag_uid, hdd_dock.get_serial_number(),
+                inventory.current_owner)
 
         elif action == '3':
-            print('Start quick wiping')
-            hdd_dock.quick_erase()
-            inventory_rest_client.add_status(
-                hdd_id, status='QUICK_WIPED',
-                extra_info='Quick wiped at {} --{}'.format(location, author))
-
-            print('Disk/ssd quick wiped')
+            manager.quick_erase()
+            inventory.flush()
 
         elif action == '4':
             manager.best_erase()
+            inventory.flush()
 
         elif action == '5':
             bay = ''
             while len(bay) == 0:
                 bay = raw_input('New server bay: ')
-            hdd = inventory_rest_client.change_bay(
-                hdd_id,
-                bay)
-            print('Server bay changed to: {}'.format(hdd['bay']))
+            inventory_rest_client.change_bay(hdd_id, bay)
+            inventory.flush()
+            print('Server bay changed to: {}'.format(inventory.bay))
 
         elif action == '6':
             health_status = ''
             while len(health_status) == 0:
-                health_status = raw_input('New health status: ')
+                health_status = raw_input('Manual health status: ')
             inventory_rest_client.add_health_status(
                 hdd_id, status=health_status, extra_info='--{}'.format(author))
-            current_health_status = health_status
+            inventory.flush()
 
         elif action == '7':
             # Print label
-            lines = [
-                'Health status: {} @ {}'.format(
-                    current_health_status.upper(),
-                    datetime.now().strftime('%Y-%m-%d')),
-                '',
-                'Serial: {}'.format(hdd_dock.get_serial_number()),
-            ]
+            today = datetime.now().strftime('%Y-%m-%d')
+            # XXX: don't use dynamic_data please :(
             total_bytes_written = [
                 x for x in dynamic_data
                 if 'Total bytes written' in x[0]][0]
@@ -339,19 +366,23 @@ def registered_disk_actions(**kwargs):
                 x for x in dynamic_data
                 if 'Total bytes read' in x[0]][0]
 
-            lines.append('Total bytes written/read: {}/{}'.format(
-                total_bytes_written[1], total_bytes_read[1]))
-
-            for item in ['Power on hours', 'Reallocated sector count']:
-                lines += [
-                    ' : '.join([x[0].rstrip(), x[1]])
-                    for x in dynamic_data if item in x[0]]
-
+            lines = [
+                'Health status : {}'.format(
+                    inventory.current_health_status.upper()),
+                '',
+                'Serial : {}'.format(hdd_dock.get_serial_number()),
+                'Total bytes written/read : {}/{}'.format(
+                    total_bytes_written[1], total_bytes_read[1]),
+                'Power on hours : {} hours'.format(
+                    hdd_dock.hwdata.power_on_hours or S_UNSET),
+                f'--{author} @ {today}',
+            ]
             labelprinter_rest_client = get_labelprinter()
             labelprinter_rest_client.print_generic_label(lines)
 
         elif action == '8':
             manager.second_best_erase()
+            inventory.flush()
 
         elif action == '9':
             break  # Quit + EJECT
@@ -364,11 +395,11 @@ def registered_disk_actions(**kwargs):
                     location, author))
             inventory_rest_client.add_location(
                 hdd_id, location='OSSO HQ: HDD security container')
+            inventory.flush()
             break
 
         elif action == 'P':
-            # FIXME: print_data is stale!
-            manager.show_summary(print_data)
+            manager.show_summary(_build_print_data())
             manager.show_commands()
 
         else:
@@ -397,21 +428,77 @@ def do_print(data, hdd_dock):
 
     for key, item in data:
         key_length = max(key_length, len(key))
-        item_length = max(item_length, len(item))
+        item_length = max(item_length, len(item or ''))
+    heading = '=' * (key_length + item_length + 3)
+    fmt = '{:%d} : {}' % (key_length,)
 
     print('DISK INFORMATION [BAY NR: {}]'.format(
         hdd_dock.docktool_bay_nr))
+    print(heading)
 
-    print('=' * (key_length + item_length + 3))
-
-    for x in data:
-        if x[0] == '=':
-            print('=' * (key_length + item_length + 3))
-        elif x[1] == '-':
-            print(x[0])
+    for key, item in data:
+        if item is None and key == '=':
+            print(heading)
+        elif item is None:
+            print(key)
         else:
-            x[0] = x[0] + ' ' * (key_length - len(x[0]))
-            print(' : '.join(x))
+            print(fmt.format(key, item))
+
+
+def build_hdd_info(hdd_dock):
+    hwdata = hdd_dock.hwdata
+
+    static_data = []
+
+    static_data.append(['Device model', hdd_dock.get_device_model()])
+    static_data.append(['Serial', hdd_dock.get_serial_number()])
+    static_data.append(
+        ['Device (port)', '{} ({})'.format(
+            hdd_dock.devname, hdd_dock.port if hdd_dock.port else 'Unknown')])
+    static_data.append(['SSD', ('yes' if hdd_dock.is_ssd() else 'no')])
+    static_data.append(
+        ['SAS (detected)', ('yes' if hdd_dock.is_sas() else 'no')])
+    static_data.append(['User Capacity', hdd_dock.get_user_capacity()])
+
+    dynamic_data = []
+
+    total_bytes_written = S_UNSET
+    total_bytes_read = S_UNSET
+    if hwdata.sector_size is not None:
+        if hwdata.lbas_written is not None:
+            total_bytes_written = human_readable_bytes(
+                hwdata.sector_size * hwdata.lbas_written)
+        if hwdata.lbas_read is not None:
+            total_bytes_read = human_readable_bytes(
+                hwdata.sector_size * hwdata.lbas_read)
+    dynamic_data.append(['Total bytes written', total_bytes_written])
+    dynamic_data.append(['Total bytes read', total_bytes_read])
+
+    def _NS(value):
+        if value is None:
+            return S_UNSET
+        return str(value)
+
+    dynamic_data.append([
+        'Power on hours',
+        '{} hours'.format(_NS(hwdata.power_on_hours))])
+    dynamic_data.append([
+        'Wear health percent', '{}% (smart: {})'.format(
+            _NS(hwdata.wear_health_percent), hwdata.smart_status)])
+    dynamic_data.append([
+        'Reallocated sector count',
+        _NS(hwdata.reallocated_sector_ct)])
+    dynamic_data.append([
+        'Reallocated event count',
+        _NS(hwdata.reallocated_event_count)])
+    dynamic_data.append([
+        'Current pending sector',
+        _NS(hwdata.current_pending_sector)])
+    dynamic_data.append([
+        'Offline uncorrectable',
+        _NS(hwdata.offline_uncorrectable)])
+
+    return static_data, dynamic_data
 
 
 def main_menu(devname):
@@ -425,59 +512,11 @@ def main_menu(devname):
         exit(1)
 
     hdd_dock = BaseStorageDevice.from_devname(devname)
-    hwdata = hdd_dock.hwdata
     sys.stdout.write('\x1b]2;DOCKTOOL DISK BAY: {}\x07'.format(
         hdd_dock.docktool_bay_nr))
 
-    static_data = []
-
-    static_data.append(['Device model', hdd_dock.get_device_model()])
-    static_data.append(['Serial', hdd_dock.get_serial_number()])
-    static_data.append(
-        ['Device (port)', '{} ({})'.format(
-            devname, hdd_dock.port if hdd_dock.port else 'Unknown')])
-    static_data.append(['SSD', ('yes' if hdd_dock.is_ssd() else 'no')])
-    static_data.append(
-        ['SAS (detected)', ('yes' if hdd_dock.is_sas() else 'no')])
-    static_data.append(['User Capacity', hdd_dock.get_user_capacity()])
-
-    dynamic_data = []
-
-    total_bytes_written = 'Notset'
-    total_bytes_read = 'Notset'
-    if hwdata.sector_size is not None:
-        if hwdata.lbas_written is not None:
-            total_bytes_written = human_readable_bytes(
-                hwdata.sector_size * hwdata.lbas_written)
-        if hwdata.lbas_read is not None:
-            total_bytes_read = human_readable_bytes(
-                hwdata.sector_size * hwdata.lbas_read)
-    dynamic_data.append(['Total bytes written', total_bytes_written])
-    dynamic_data.append(['Total bytes read', total_bytes_read])
-
-    def _NS(value):
-        if value is None:
-            return 'Notset'
-        return str(value)
-
-    dynamic_data.append([
-        'Power on hours',
-        '{} hours'.format(_NS(hwdata.power_on_hours))])
-    dynamic_data.append([
-        'Wear leveling count',
-        _NS(hwdata.wear_leveling_count)])
-    dynamic_data.append([
-        'Reallocated sector count',
-        _NS(hwdata.reallocated_sector_ct)])
-    dynamic_data.append([
-        'Reallocated event count',
-        _NS(hwdata.reallocated_event_count)])
-    dynamic_data.append([
-        'Current pending sector',
-        _NS(hwdata.current_pending_sector)])
-    dynamic_data.append([
-        'Offline uncorrectable',
-        _NS(hwdata.offline_uncorrectable)])
+    # XXX: build_hdd_info() is a quick hack..
+    static_data, dynamic_data = build_hdd_info(hdd_dock)
 
     inventory_rest_client = InventoryRESTClient(
         settings.DASHBOARD_BASE_URL)
