@@ -42,6 +42,9 @@ class BaseStorageDevice(object):
         self.port = self._get_port()
         self.docktool_bay_nr = self._get_docktool_bay_nr(self.port)
 
+    def flush(self):
+        self.devtype, self.hwdata = self._get_hwdata(self.devname)
+
     def get_device_model(self):
         # "Samsung SSD 850 PRO 256GB" or ValueError
         return self.hwdata.device_model
@@ -66,9 +69,9 @@ class BaseStorageDevice(object):
         # "ACS-2, ATA8-ACS T13/1699-D revision 4c" or "unknown"
         return self.hwdata.ata_version
 
-    def get_smart_support(self):
-        # "Enabled" or "unknown" (or ...?)
-        return self.hwdata.smart_support
+    def get_smart_status(self):
+        # "PASSED" or "FAILED!" or "Enabled" or "MISSING"
+        return self.hwdata.smart_status
 
     def is_sas(self):
         return 'sas' in self.hwdata.information.get(
@@ -331,7 +334,7 @@ class NvmeStorageDevice(BaseStorageDevice):
         else:
             print('WARNING: Skipping secure block erase because DEBUG')
         return True, None
-    wipe_using_block_erase.name = 'nvme format (poor, slow)'
+    wipe_using_block_erase.name = 'nvme format (poor)'
     wipe_using_block_erase.type = 'builtin'  # ???
 
     def wipe_using_block_sanitize(self):
@@ -341,7 +344,7 @@ class NvmeStorageDevice(BaseStorageDevice):
         else:
             print('WARNING: Skipping secure block sanitize because DEBUG')
         return True, None
-    wipe_using_block_sanitize.name = 'nvme block sanitize (best, slow)'
+    wipe_using_block_sanitize.name = 'nvme block sanitize (better)'
     wipe_using_block_sanitize.type = 'builtin'  # ???
 
     def wipe_using_crypto_erase(self):
@@ -351,7 +354,7 @@ class NvmeStorageDevice(BaseStorageDevice):
         else:
             print('WARNING: Skipping secure crypto erase because DEBUG')
         return True, None
-    wipe_using_crypto_erase.name = 'nvme crypto erase (good, fast)'
+    wipe_using_crypto_erase.name = 'nvme crypto erase (better)'
     wipe_using_crypto_erase.type = 'builtin'  # ???
 
     def wipe_using_crypto_sanitize(self):
@@ -361,7 +364,7 @@ class NvmeStorageDevice(BaseStorageDevice):
         else:
             print('WARNING: Skipping secure crypto sanitize because DEBUG')
         return True, None
-    wipe_using_crypto_sanitize.name = 'nvme crypto sanitize (best, fast)'
+    wipe_using_crypto_sanitize.name = 'nvme crypto sanitize (best)'
     wipe_using_crypto_sanitize.type = 'builtin'  # ???
 
     def _get_secure_erase_options(self):
